clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Read and process friction data and compute/save friction curve 

% Version: 13-10-2022
% Matlab version: R2021a


%% initialisation
figure(1), clf(1)
mat_type = 1; % 1: C/PEEK, 2: C/LM-PAEK
mod_coeff = 0.90; %threshold for modulus fit, end; max difference local and average modulus
rel_time = 28.5; % time after sliding stopped for relaxation curve

if mat_type == 1 % C/PEEK material
    serie_range = [2 3 15]; % series of experiments (relates to unique number of specimen)
    series{1} = [1 2 3 4 5 6]; % experiments in series/range (relates to unique number of specimen)
    series{2} = [7 8];
    series{3} = [7 8 9 17 18];
elseif mat_type == 2 % C/LM-PAEK material
    serie_range = [17]; % series of experiments (relates to unique number of specimen)
    series{1} = [10 11 12 34 31 7 8 9 4 5 6]; % experiments in range (relates to unique number of specimen)
end

%% Go through the measurements

for serie_i = 1:length(serie_range) % go through series (of experiments)
    
    serie = serie_range(serie_i);
    range_meas = series{serie_i};
    
    for meas_i = 1:length(range_meas) % go through measurements within specific series

        meas = range_meas(meas_i);

        %% Load data
        if mat_type == 1 % C/PEEK
            cd ../FT_C-PEEK_15kPa_385degC
        elseif mat_type == 2 % C/LM-PAEK
            cd ../FT_C-LMPAEK_15kPA_365degC
        end
        file_find = dir(['*_',num2str(serie),'-',num2str(meas),'.*']);
        file_name = file_find.name;
        
        if file_name(end-2:end) == 'txt'  % data logged on laptop (different format, higher sampling rate)
            data = readmatrix(file_name,'NumHeaderLines',1);
            fprintf(['File: ', file_find.name,'\n'])
            cd ../Scripts
            
            t = data(:,1); % time [s]
            d = data(:,6) - data(1,6);  % displacement [mm]
            force_corr = mean(data(end-200:end,5)); % end value (end of experiment)
            f_pull = data(:,5) - force_corr; % pull force, corrected for end value [N}
            
        else    % data logged on friction tester PC (lower sampling rate)
            fid = fopen(file_name);
            C = textscan(fid,'%f');
            data = zeros(int32(length(C{1})/13),13);
            for i = 0:length(C{1})-1
                i_row = 1+floor(i/13);
                i_col = 1+mod(i,13);
                data(int32(i_row),int32(i_col)) = C{1}(i+1);
            end
            fprintf(['File: ', file_find.name,'\n'])
            fclose(fid);
            cd ../Scripts
            
            t = data(:,2) - data(1,2); % time [s]
            d = data(1,13) - data(:,13); % displacement [mm]
            force_corr = mean(data(end-5:end,6)); % end value (end of experiment)
            f_pull = data(:,6) - force_corr; % pull force, corrected for end value [N]
            
        end       
        
        %% Analysis

        %% index start/end using displacement data, mean velocity, shear stress
        
        % velocity from file name
        name_u = find(file_find.name=='u');
        name_u_end = name_u + 1;
        while file_find.name(name_u_end) ~= '_'
            name_u_end = name_u_end + 1;
        end
        vel = str2double(file_find.name(name_u+1:name_u_end-1)); % velocity from file name

        % determine window for smoothening based on velocity form file name
        if file_name(end-2:end) == 'txt'  % data logged on laptop (higher sampling rate; larger window for smoothening)
            if vel <= 1
                window = 525;
            elseif vel <= 3
                window = 75;
            elseif vel <= 5
                window = 100;
            elseif vel <= 11
                window = 40;
            elseif vel <= 15
                window = 20;
            elseif vel <= 30
                window = 20;
            else
                window = 15;
            end
        else
            if vel <= 1
                window = 100;
            elseif vel <= 3
                window = 75;
            elseif vel <= 5
                window = 50;
            elseif vel <= 8
                window = 25;
            elseif vel <= 15
                window = 15;
            else
                window = 15;
            end
        end

        % smoothing data
        d_filt = smoothdata(d,'gaussian',window); % filter data

        v_vec = [0]; % velocity vector
        a_vec = [0]; % acceleration vector
        for i = 2:length(t)-1
            dt = t(i+1) - t(i);
            v = (d_filt(i+1)-d_filt(i-1))/(2*dt);
            a = (d_filt(i+1)-2*d_filt(i)+d_filt(i-1))/(dt^2);
            v_vec(i) = v;
            a_vec(i) = a;
        end
        v_vec(length(t)) = v;
        a_vec(length(t)) = a;
        
        % determine global start and end of friction response
        i_glob = 1; % global end range of start point
        while d(i_glob) < 1 % deformation below 1 mm
            i_glob = i_glob + 1;
        end
        
        i_glob_9 = i_glob; % global end range of start point, also global start range for end point
        while d(i_glob_9) < 9 % deformation around 9 mm
            i_glob_9 = i_glob_9 + 1;
        end
        
        i_glob_start = i_glob; % find around i_glob also global start range for start point
        while t(i_glob_start) > t(i_glob)-(2/vel*60)  % move 2 mm in time earlier based on velocity in experiment
            i_glob_start = i_glob_start - 1;
        end
                
        i_glob_end = i_glob;
        if mat_type == 1
            while t(i_glob_end) < t(i_glob) + d(end)/vel*60 % find index for time after full displacement
                i_glob_end = i_glob_end + 1;
            end
        else
            while t(i_glob_end) < t(i_glob) + min(11.5,max(d))/vel*60 % find index for time after 11.5 mm displacement (not d(end) because of additional movement after testing...)
                i_glob_end = i_glob_end + 1;
            end           
        end           
                
        idx_start = find(a_vec(i_glob_start:i_glob)==max(a_vec(i_glob_start:i_glob)))+i_glob_start-1;
        idx_end = find(a_vec(i_glob_9:i_glob_end)==min(a_vec(i_glob_9:i_glob_end)))+i_glob_9-1;

        % mean velocity
        V_mean = (d(idx_end)-d(idx_start))/(t(idx_end)-t(idx_start)); % mm/s
        
        % shear stress
        tau = f_pull/2500/2*1e3; % [N/mm] -> MPa, times 1e3 results in [kPa] (division by 2, as 2 slip interfaces are present)
        
        %% Relaxation
        % used to obtain end value after full relaxation (yield stress or constant contribution)
        
        % start and end indices
        idx_f_rel_start = idx_end - 10; % global start range for point of relaxation start
        idx_f_rel_end = idx_end + 10;   % global end range for point of relaxation start
        
        % determine derivative to identify start of relaxation (drop in
        % shear stress)
        dtau_dt_sec_vec = [];
        for i_rel = idx_f_rel_start:idx_f_rel_end
            dt = mean([t(i_rel+1)-t(i_rel),t(i_rel)-t(i_rel-1)]);  % avg dt within stencils for derivative of shear stress wrt time
            dtau_dt_sec = (tau(i_rel+1)-2*tau(i_rel)+tau(i_rel-1))/(dt^2); % compute derivative
            dtau_dt_sec_vec = [dtau_dt_sec_vec, dtau_dt_sec]; % store
        end
        
        idx_rel_start = find(dtau_dt_sec_vec==min(dtau_dt_sec_vec))+idx_f_rel_start-2; % minus 1: to obtain correct pos, extra minus 1: to start before actual sharp decrease
        
        % find end of relaxation
        idx_rel_end = idx_rel_start;
        while t(idx_rel_end) < t(idx_rel_start)+rel_time  
            idx_rel_end = idx_rel_end + 1;
        end
        tau_rel = tau(idx_rel_start:idx_rel_end); 
        d_tau_end = mean(tau_rel(end-10:end)); % end value of shear stress after ~full relaxation
          
        %% modulus fit
        % used to align friction curves
        
        [t,d] = mod_fit_v1(idx_start,idx_end,t,d,tau,mod_coeff,20);
        
        %% Save curve
        if mat_type == 1
            data_name = "curve_T_C-PEEK_p15_u" + num2str(vel) + "_T385_" + num2str(serie) + "-" + num2str(meas); 
        elseif mat_type == 2
            data_name = "curve_T_C-LM-PAEK_p15_u" + num2str(vel) + "_T365_" + num2str(serie) + "-" + num2str(meas); 
        end
        d_save = d(idx_start:idx_end);
        t_save = t(idx_start:idx_end);
        tau_save = tau(idx_start:idx_end);
        
        cd ../Friction_curves
        %save(data_name,'d_save','t_save','tau_save','d_tau_end','V_mean');
        cd ../Scripts
        
        %% Show data

        % friction response
        figure(1), hold on
        plot(d_save,tau_save,'-')
        xlabel('Displacement [mm]')
        ylabel('Shear stress [kPa]')
        box on
        grid on

    end
end