clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Run certain height profile with 1D shear rate approach with White-Metzner
% to compute VE response with and without wall slip using state-rate
% model for slip evolution, different matrix interlayer thickness
% distributions to address sensitivity for h(x)

% Version:
%   13-10-2022

% Matlab version: R2021a/R2022b

%% initialisation

mat_type = 2; %1: C/PEEK, 2: C/LM-PAEK
V_vec = 125; % sliding velocity, mm/min (for modeling)
slip = 1; % 1: slip effect, else no slip condition
dt = 0.01; % time step
tau_cr_vec = [35e3 55e3]; % critical shear stress for onset of strong slip
G_WM_vec = [500 400]; % parameter for White-Metzner (stiffness, controls start-up slope)

% state-rate model parameters and initial values
k_min_vec = [0.75 1.8]; 
a_slip_vec = [4.6 0.25]; 
m_slip_vec = [1.95 4.2];
Z_e_0 = 1.0;
Z_d_0 = 0.0;
Z_o_0 = 0.0;

% viscosity parameters for Cross model (from rheometry data)
eta_0_Cross_vec = [588, 499]; % zero shear viscosity
tau_0_Cross_vec = [6.05e4, 2.53e5]; 
n_Cross_vec = [0.38, 0.42];  
const_contr_vec = [1.8, 0.5]; % constant contribution (fiber-fiber interaction)

% figures
figure(1), clf(1), hold on

% parameters based on material type
G_WM = G_WM_vec(mat_type);                % WM parameter, start-up behavior
tau_cr = tau_cr_vec(mat_type);            % Slip parameter, onset for strong slip
k_min = k_min_vec(mat_type);              % State-rate parameter, minimum rate 
a_slip = a_slip_vec(mat_type);            % State-rate parameter, V-dependency
m_slip = m_slip_vec(mat_type);            % State-rate parameter, V-dependency
eta_0_Cross = eta_0_Cross_vec(mat_type);  % Cross model parameter
tau_0_Cross = tau_0_Cross_vec(mat_type);  % Cross model parameter
n_Cross = n_Cross_vec(mat_type);          % Cross model parameter
tau_y = const_contr_vec(mat_type);        % residual stress after full relaxation (not from VE response, yield stress)

% Cross model description
eta_func = @(V,h) ((eta_0_Cross)./(1+(eta_0_Cross.*(V./h)./tau_0_Cross).^(1-n_Cross)));

% find critical shear rate with viscous shear stress = tau_cr (for V_s,infty)
fun_find_gam = @(gam) ((eta_0_Cross)./(1+(eta_0_Cross.*(gam)./tau_0_Cross).^(1-n_Cross))).*(gam)-tau_cr; % function for gam_dot for which viscous response (Cross model) equals critical shear stress 
gam_dot_cr = fzero(fun_find_gam,tau_cr/eta_0_Cross*2); % solve for gam_dot

%% Compute shear flow model with slip evolution

% choose ply-ply matrix interlayer thickness profiles
profile = "profile_gen_dist_59_multiple.mat"; % profile 3 shows average response (based on results in earlier paper for prediction friction characteristics)

% load profiles
load(profile);

for j = [3,5,9] % pick 3rd, 5th, and 9th matrix interlayer thickness distribution (resulting in low, average (default), and high response)

    % load matrix interlayer thickness distribution from ten in dataset
    x_profile = store_profile{j}(1,:)*1e3; % measured profile mm, width
    y_profile = store_profile{j}(2,:)*1e3; % measured profile mm, height

    V_sliding = V_vec/60; % mm/s
    t_e = 10/V_sliding; % end time for numerical time integration
    t = 0:dt:t_e;  % time vector
    d = t*V_sliding; % displacement vector
    tau_store = zeros(length(x_profile)-1,length(t)); % store tau curves per width increment
    slip_count = 0; % for percentage of slip (at certain velocity)
    
    % go through ply-ply matrix interlayer thickness profile
    for i = 1:length(x_profile)-1
        dx = x_profile(i+1)-x_profile(i); % width of small segment, used for weighting if not equidistant
        dy = 0.5*(y_profile(i)+y_profile(i+1)); % height of small segment
        if dy == 0 % no shear rate, thus zero shear stress
            tau_i = zeros(1,length(t));
        else
            
            % White-Metzner DE for Runge-Kutta numerical time integration
            f = @(tau,Vs) (G_WM.*((V_sliding-2*Vs)/dy) - (G_WM./eta_func(V_sliding-2*Vs,dy)).*tau);
           
            % initialisation
            tau_i = zeros(1,length(t)); % store shear stresses
            Vs = zeros(1,length(t)); % store slip velocities
            strong_slip = 0;
            Z_e = ones(1,length(t)).*Z_e_0;  % store average state of active chains (or entangled)
            Z_d = ones(1,length(t)).*Z_d_0;  % store average state of disentangled chains
            Z_o = ones(1,length(t)).*Z_o_0;  % store average state of steady chains (or oriented) 

            for k = 1:length(t)-1  % go through time vector (within small segment of ply-ply interface)        

                % Runge-Kutta for numerical time integration
                k1 = f(tau_i(k),Vs(k));
                k2 = f(tau_i(k)+0.5*dt*k1,Vs(k));
                k3 = f(tau_i(k)+0.5*dt*k2,Vs(k));
                k4 = f(tau_i(k)+dt*k3,Vs(k));
                
                % shear stress at next time step
                tau_i(k+1) = tau_i(k) + (1/6)*dt*(k1+2*k2+2*k3+k4);
                
                % strong slip effect (with slip evolution)
                if (slip == 1 && tau_i(k) >= tau_cr) || strong_slip == 1
                    strong_slip = 1;
                    gam_dot = V_sliding/dy; % local shear rate in small segment
                  
                    % stationary slip velocity (V_s,infty denoted here a Vs_cr) such that true shear rates results in critcal shear rate                    
                    Vs_cr = 0.5*(V_sliding-gam_dot_cr*dy); % max slip velocity (to get tau_cr)
                    Vs_cr  = max(Vs_cr,0); % only if gam_dot > gam_dot_cr!
                   
                    % rates (based on mode and applied rate) 
                    k_max = a_slip.*((V_sliding)).^m_slip + k_min; % max rate
                    k_d = k_min + (k_max-k_min)*(1-Z_e(k)); % disentangling rate
                    k_e = k_max - (k_max-k_min)*(1-Z_e(k)); % reentanglement rate
                    k_o = k_d; % conformation or orientation rate
                    
                    % integration of states (based on rates, forward Euler)
                    Z_e(k+1) = Z_e(k) - k_d*Z_e(k)*dt + k_e*Z_d(k)*dt;   
                    Z_d(k+1) = Z_d(k) + k_d*Z_e(k)*dt - k_e*Z_d(k)*dt - k_o*Z_d(k)*dt;
                    Z_o(k+1) = Z_o(k) + k_o*Z_d(k)*dt;
                    
                    % safety: states within 0 and 1
                    Z_e(k+1) = min(Z_e(k+1),1);
                    Z_d(k+1) = min(Z_d(k+1),1);
                    Z_o(k+1) = min(Z_o(k+1),1);
                    Z_e(k+1) = max(Z_e(k+1),0);
                    Z_d(k+1) = max(Z_d(k+1),0);
                    Z_o(k+1) = max(Z_o(k+1),0);

                    % slip velocity: based on average steady (or orientated) state with prefactor (stationary slip velocity)
                    Vs(k+1) = Z_o(k+1)*Vs_cr;
                    
                end
            end
            
        end
        tau_store(i,:) = tau_i*dx; %shear stress small portion times width(weighting), [Pamm]
        
        if strong_slip == 1
            slip_count = slip_count + 1; % for precentage of slip
        end       
    end
    tau_avg = sum(tau_store,1)/(x_profile(end)-x_profile(1));

    figure(1), hold on
    if j == 3 % 3rd profile is default profile (average response)
        avg_curve = [d; tau_avg*1e-3+tau_y]; % VE response + constant contribution (yield stress)
    elseif j == 5
        upper_curve = [d; tau_avg*1e-3+tau_y]; % highest response
    elseif j == 9
        lower_curve = [d; tau_avg*1e-3+tau_y]; % lowest response (of matrix interlayer thickness distributions in datatset)
    end
end

%% Figure options

figure(1), hold on
plot(avg_curve(1,:),avg_curve(2,:),'--k','displayname',['Default $h(x)$'],'linewidth',1.0)
plt = fill([upper_curve(1,:), fliplr(lower_curve(1,:))],[upper_curve(2,:), fliplr(lower_curve(2,:))],[0.5 0.5 0.5],'LineStyle','none','facecolor',[0.0 0.45 0.74],'displayname',['Bounds for' newline 'other $h(x)$']);
%set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
set(plt, 'facealpha', 0.3)
plt = plot(avg_curve(1,:),avg_curve(2,:),'-k','displayname',['Default $h(x)$'],'linewidth',1.0); 
set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
legend('location','northeast','interpreter','latex','fontsize',8,'box','off')
xlabel('Displacement, $d$ [mm]','interpreter','latex','fontsize',8)
ylabel('Shear stress, $\tau$ [kPa]','interpreter','latex','fontsize',8)
set(gca,'fontsize',8)
axis([0 10 0 150])

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'renderer','painters','units','centimeters','position',[5,5,width,0.75*width])
grid on
box on

