clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Show evolution states and slip velocity for different rates and matrix 
% interlayer thicknesses

% Version:
%   2-11-2022

% Matlab version: R2021a

%% initialisation

mat_type = 1; %1: C/PEEK, 2: C/LM-PAEK
mode = 1; % 1: plot evolution of states with time 2: plot Vs/V with t for different h and constant V, 3: plot Vs/V with t for different V and constant h
dt = 0.01; % time step
t_e = 5; % end time for numerical time integration
t = 0:dt:t_e;  % time vector
tau_cr_vec = [35e3 55e3]; % critical shear stress for onset of strong slip
G_WM_vec = [500 400]; % parameter for White-Metzner (stiffness, controls start-up slope)

% state-rate model parameters and initial values
k_min_vec = [0.75 1.8]; 
a_slip_vec = [4.6 0.25]; 
m_slip_vec = [1.95 4.2];
Z_e_0 = 1.0;
Z_d_0 = 0.0;
Z_o_0 = 0.0;

% viscosity parameters for Cross model (from rheometry data)
eta_0_Cross_vec = [588, 499]; % zero shear viscosity
tau_0_Cross_vec = [6.05e4, 2.53e5]; 
n_Cross_vec = [0.38, 0.42]; 
const_contr_vec = [1.8, 0.5]; % constant contribution or yield stress (fiber-fiber interaction)

% figures
figure(1), clf(1), hold on

% parameters based on material type
G_WM = G_WM_vec(mat_type);                % WM parameter, start-up behavior
tau_cr = tau_cr_vec(mat_type);            % Slip parameter, onset for strong slip
k_min = k_min_vec(mat_type);              % State-rate parameter, minimum rate 
a_slip = a_slip_vec(mat_type);            % State-rate parameter, V-dependency
m_slip = m_slip_vec(mat_type);            % State-rate parameter, V-dependency
eta_0_Cross = eta_0_Cross_vec(mat_type);  % Cross model parameter
tau_0_Cross = tau_0_Cross_vec(mat_type);  % Cross model parameter
n_Cross = n_Cross_vec(mat_type);          % Cross model parameter
tau_y = const_contr_vec(mat_type);        % residual stress after full relaxation (not from VE response, yield stress)

% Cross model description 
eta_func = @(V,h) ((eta_0_Cross)./(1+(eta_0_Cross.*(V./h)./tau_0_Cross).^(1-n_Cross)));

% find critical shear rate with viscous shear stress = tau_cr (for V_s,infty)
fun_find_gam = @(gam) ((eta_0_Cross)./(1+(eta_0_Cross.*(gam)./tau_0_Cross).^(1-n_Cross))).*(gam)-tau_cr; % function for gam_dot for which viscous response (Cross model) equals critical shear stress 
gam_dot_cr = fzero(fun_find_gam,tau_cr/eta_0_Cross*2); % solve for gam_dot

%% Compute shear flow model with slip evolution

% choose ply-ply matrix interlayer thickness profiles
profile = "profile_gen_dist_59_multiple.mat"; % profile 3 shows average response (based on results in earlier paper for prediction friction characteristics)

% load profiles
load(profile);

% pick profile 3 (from 10 profiles in dataset)
x_profile = store_profile{3}(1,:)*1e3; % measured profile mm, width
y_profile = store_profile{3}(2,:)*1e3; % measured profile mm, height

if mode == 1
    % V
    V_vec = [60]; % sliding velocity, mm/min (for modeling)
    % h
    y_profile = [1.5e-3 1.5e-3];
    x_profile = 1:length(y_profile);
    % longer time vector
    t_e = 10; % end time for numerical time integration
    t = 0:dt:t_e;  % time vector
    
elseif mode == 2
    % V
    V_vec = [60]; % sliding velocity, mm/min (for modeling)
    % h
    y_profile = [0 0.001 0.002 0.004 0.005];
    x_profile = 1:length(y_profile);
    
elseif mode == 3
    % V
    V_vec = [125 60 40 25];
    % h
    y_profile = [1.5e-3 1.5e-3];
    x_profile = 1:length(y_profile);
end

for j = 1:length(V_vec) % go through velocities
    V_sliding = V_vec(j)/60; % mm/s
    d = t*V_sliding; % displacement vector
    tau_store = zeros(length(x_profile)-1,length(t)); % store tau curves per width increment  
    
    % go through ply-ply matrix interlayer thickness profile
    for i = 1:length(x_profile)-1
        dx = x_profile(i+1)-x_profile(i); % width of small segment, used for weighting if not equidistant
        dy = 0.5*(y_profile(i)+y_profile(i+1)); % height of small segment
        if dy == 0 % no shear rate, thus zero shear stress
            tau_i = zeros(1,length(t));
            Vs = zeros(1,length(t)); % all slip velocities zero
            Vs_cr = 0; % no slip velocity
        else
            
            % White-Metzner DE for Runge-Kutta numerical time integration
            f = @(tau,Vs) (G_WM.*((V_sliding-2*Vs)/dy) - (G_WM./eta_func(V_sliding-2*Vs,dy)).*tau);
           
            % initialisation
            tau_i = zeros(1,length(t)); % store shear stresses
            Vs = zeros(1,length(t)); % store slip velocities
            strong_slip = 0;
            Z_e = ones(1,length(t)).*Z_e_0;  % store average state of active chains (or entangled)
            Z_d = ones(1,length(t)).*Z_d_0;  % store average state of disentangled chains
            Z_o = ones(1,length(t)).*Z_o_0;  % store average state of steady chains (or oriented) 

            for k = 1:length(t)-1  % go through time vector (within small segment of ply-ply interface)        

                % Runge-Kutta for numerical time integration
                k1 = f(tau_i(k),Vs(k));
                k2 = f(tau_i(k)+0.5*dt*k1,Vs(k));
                k3 = f(tau_i(k)+0.5*dt*k2,Vs(k));
                k4 = f(tau_i(k)+dt*k3,Vs(k));
                
                % shear stress at next time step
                tau_i(k+1) = tau_i(k) + (1/6)*dt*(k1+2*k2+2*k3+k4);
                
                % strong slip effect (with slip evolution)
                if tau_i(k) >= tau_cr || strong_slip == 1
                    strong_slip = 1;
                    gam_dot = V_sliding/dy; % local shear rate in small segment
                  
                    % stationary slip velocity (V_s,infty denoted here a Vs_cr) such that true shear rates results in critcal shear rate                    
                    Vs_cr = 0.5*(V_sliding-gam_dot_cr*dy); % max slip velocity (to get tau_cr)
                    Vs_cr  = max(Vs_cr,0); % only if gam_dot > gam_dot_cr!
                   
                    % rates (based on applied rate) 
                    k_max = a_slip.*((V_sliding)).^m_slip + k_min; % max rate
                    k_d = k_min + (k_max-k_min)*(1-Z_e(k)); % disentangling rate
                    k_e = k_max - (k_max-k_min)*(1-Z_e(k)); % reentanglement rate
                    k_o = k_d; % conformation or orientation rate
                    
                    % integration of states (based on rates, forward Euler)
                    Z_e(k+1) = Z_e(k) - k_d*Z_e(k)*dt + k_e*Z_d(k)*dt;   
                    Z_d(k+1) = Z_d(k) + k_d*Z_e(k)*dt - k_e*Z_d(k)*dt - k_o*Z_d(k)*dt;
                    Z_o(k+1) = Z_o(k) + k_o*Z_d(k)*dt;
                    
                    % safety...
                    Z_e(k+1) = min(Z_e(k+1),1);
                    Z_d(k+1) = min(Z_d(k+1),1);
                    Z_o(k+1) = min(Z_o(k+1),1);
                    Z_e(k+1) = max(Z_e(k+1),0);
                    Z_d(k+1) = max(Z_d(k+1),0);
                    Z_o(k+1) = max(Z_o(k+1),0);

                    % slip velocity: based on fraction of steady (or orientated) state
                    Vs(k+1) = Z_o(k+1)*Vs_cr;
                else
                    Vs_cr = 0; % no slip
                end
            end                 
        end
        tau_store(i,:) = tau_i*dx; %shear stress small portion times widht (weighting), [Pamm]
        
        if mode == 1 % show evolution of states
            plot(t,Z_e,'-k','color',[0 0 0],'displayname','$Z^{\textup{a}}$')
            plot(t,Z_d,'--k','color',[0 0 0],'displayname','$Z^{\textup{d}}$')
            plot(t,Z_o,'-.k','color',[0 0 0],'displayname','$Z^{\textup{s}}$')
        elseif mode == 2 % show evolution Vs with t for certain h
            plot(t,Vs/V_sliding,'displayname',[num2str(dy*1e3,'%0.1f') ' $\mu$m'])
        elseif mode == 3 % show evolution of Vs with t for certain V
            plot(t,Vs/V_sliding,'displayname',[num2str(V_vec(j),'%0.0f') ' mm/min'])
        end
            
    end
    tau_avg = sum(tau_store,1)/(x_profile(end)-x_profile(1));
    
end

%% Figure options
figure(1)

if mode == 1
    axis([0 10 0 1])
    xlabel('Time, t [s]','interpreter','latex','fontsize',8)
    ylabel('States, $Z^{i}$ [-]','interpreter','latex','fontsize',8)
    legend('location','northeast','interpreter','latex','fontsize',8,'box','off')
elseif mode == 2
    text(0.1,0.475,['$V$ = 60 mm/min'],'interpreter','latex','fontsize',8)
    xlabel('Time, $t$ [s]','interpreter','latex','fontsize',8)
    ylabel('Slip velocity ratio, $V_{\textup{s}}/V$ [-]','interpreter','latex','fontsize',8)
    legend('box','off','location','southeast','interpreter','latex','fontsize',8)
elseif mode == 3
    text(0.1,0.475,['$h$ = 1.5 $\mu$m'],'interpreter','latex','fontsize',8)
    xlabel('Time, $t$ [s]','interpreter','latex','fontsize',8)
    ylabel('Slip velocity ratio, $V_{\textup{s}}/V$ [-]','interpreter','latex','fontsize',8)
    legend('box','off','location','southeast','interpreter','latex','fontsize',8)
end
    
box on
grid on
set(gca,'fontsize',8)

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'units','centimeters','position',[5,5,width,0.75*width])
