clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Plot plate-plate rheometry data on LM-PAEK AE250P and fit Cross model and
% Arrhenius equation to describe viscosity as function of shear rate and 
% temperature

% Version: 13-10-2022
% Matlab version: R2021a

%% Initialisation
file_name = 'LM_PAEK_rheometry_data';

figure(1), clf(1)


%% Load data and take average per temperature (if required)

cd ../

% data at 365 degC
sheet_1 = 'LMPAEK_19_FREQ_1';
data = xlsread(file_name,sheet_1);
eta_1 = data(5:end,3); % complex viscosity, Pa*s
ang_freq = data(5:end,2); % in rad/s
sheet_2 = 'LMPAEK_19_FREQ_2';
data = xlsread(file_name,sheet_2);
eta_2 = data(5:end,3); % complex viscosity, Pa*s
sheet_3 = 'LMPAEK_21_FREQ_2';
data = xlsread(file_name,sheet_3);
eta_3 = data(5:end,3); % complex viscosity, Pa*s

% compute average viscosity from 3 measurements
eta_365 = mean([eta_1, eta_2, eta_3],2); 

%% Cross model fit

% function fit: Cross viscosity model 
eta_cross_fun = @(eta_0,tau_0,n,gam_dot) (eta_0)./(1+(eta_0.*gam_dot./tau_0).^(1-n));
gam_dot_cross_vec = logspace(-1,4,200); % can be used for plotting

% fit at 365 degC data
[A_365, fval_365] = fminsearch(@(a) norm(eta_cross_fun(a(1),a(2),a(3),ang_freq)-eta_365),[500,6e4,0.4]);
eta_fit_365 = A_365(1);
tau_fit_365 = A_365(2);
n_fit_365 = A_365(3);

% compute model response
eta_cross_fun_365 = eta_cross_fun(eta_fit_365,tau_fit_365,n_fit_365,gam_dot_cross_vec);

%% Plot Cross fit

% plot data
figure(1), clf(1), hold on

% exp. data
plot(ang_freq,eta_365,'k^','markersize',6,'displayname',['Plate-plate meas. 365 $^\circ$C'],'MarkerIndices',1:2:length(ang_freq))

% Cross model fit
plot(gam_dot_cross_vec,eta_cross_fun_365,'-k','linewidth',2.0,'displayname',['Cross model fit, $\eta_0$ = ', num2str(eta_fit_365,'%0.f'),' Pas, ' newline ' $\tau^*$ = ', num2str(tau_fit_365,'%.3g'), ' Pa, $n$ = ', num2str(n_fit_365,'%0.2f')])

%% Plot viscosity as provided by Victrex (capillary rheometry)

% determined shear rate from provided graph (capillary)
shear_rate = [100 200 500 1e3 2e3 5e3 10e3]; % 1/s
visc_360 = [240 220 200 175 150 105 72]; % Pas (read from graph), 360 degC
visc_380 = [190 175 156 141 121 89 65]; % Pas (read from graph), 360 degC
visc_365 = zeros(1,length(visc_360));

for i = 1:length(visc_365)
    visc_365(i) = interp1([360 380],[visc_360(i) visc_380(i)],365);
end

figure(1), hold on
plot(shear_rate,visc_365,'kx','markersize',6,'displayname',['Capillary data, $365 ^\circ$C'])

%% Figure options

yticks([50 100 200 300 500 700])
xticks([1 1e1 1e2 1e3 1e4])
box on
grid on
legend('location','southwest','interpreter','latex','fontsize',8,'box','off')
xlabel('Shear rate, $\dot{\gamma}$ [1/s]','interpreter','latex','fontsize',8)
ylabel('Viscosity, $\eta$ [Pas]','interpreter','latex','fontsize',8)
set(gca,'XScale','log','YScale','log','fontsize',8)

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'units','centimeters','position',[5,5,width,0.75*width])
axis([1e-1 1e4 50 700])
grid on
box on