clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Reproduce curves based on rheometer data PEEK 150P from Marten van der Werff (TPRC)
% fit with Cross model and use Arrhenius relation for zero shear viscosity
% at 385 degC (with other Cross model parameters from 380 degC fit)

% Version: 13-10-2022
% Matlab version: R2021a


%% Experimental data

ang_freq = [0.5 0.629326621 0.792393027 0.998003992 1.255965838 1.581027668 1.99044586 2.505637685 3.154574132 3.971405878 5,...
    6.293266205 7.923930269 9.98003992 12.55965838 15.81027668 19.9044586 25.05637685 31.54574132 39.71405878 50 62.93266205,...
    79.23930269 99.8003992 125.5965838 158.1027668 199.044586];  % angular frequency
eta_380 = [619 617.4 616.8 616.1 613.4 610.1 605.4 597.9 589.3 579.4 568.4 555.1 540.3 523.9 506.5 488 468.3 447.3 426.1 404.5 383.9,...
    362.8 341.4 320.2 299.3 280.5 259];
eta_390 = [507.8 501.9 497.5 493.7 489.4 485.7 480.6 475.4 469.7 463 455.1 445.8 435.2 423.4 410.8 397.8 383.2 367.6 351.5 335.2,...
    318.5 301.6 284.7 268.3 251.8 235.2 219.1];

%% Cross model viscosity

% functie fit: Cross viscosity model 
eta_cross_fun = @(eta_0,tau_0,n,gam_dot) (eta_0)./(1+(eta_0.*gam_dot./tau_0).^(1-n));
gam_dot_cross_vec = logspace(-0.5,2.3,200); % can be used for plotting

% fit 380 and 390 degC data
[A_380, fval_380] = fminsearch(@(a) norm(eta_cross_fun(a(1),a(2),a(3),ang_freq)-eta_380),[600,6e4,0.4]);
eta_fit_380 = A_380(1);
tau_fit_380 = A_380(2);
n_fit_380 = A_380(3);

[A_390, fval_390] = fminsearch(@(a) norm(eta_cross_fun(a(1),a(2),a(3),ang_freq)-eta_390),[600,6e4,0.4]);
eta_fit_390 = A_390(1);
tau_fit_390 = A_390(2);
n_fit_390 = A_390(3);

% use fit parameters at 380 degC
tau_fit_base = tau_fit_380;
n_fit_base = n_fit_380;

%% Arrhenius fit for T-dependency of zero shear viscosity

% fitted zero shear viscosity from Cross model
eta_0_vec = [eta_fit_380 eta_fit_390]; % fitted eta_0 values
T_vec = [380 390];

% Arrhenius equation eta = D1 exp (E/R(1/T-1/T_ref))
R = 8.31; % universal gas constant, J/(mol K)
T_ref = 380+273.15;
eta_0_ref = eta_fit_380;
Arrhenius_fun = @(eta_0_ref,E,T_ref,T) eta_0_ref.*exp((E./R).*((1./T)-(1./T_ref)));
[A, fvalA] = fminsearch(@(a) norm(Arrhenius_fun(eta_0_ref,a(1),T_ref,T_vec+273.15)-eta_0_vec),[5e4]);
E_fit = A(1);

eta_fit_385 = Arrhenius_fun(eta_0_ref,E_fit,T_ref,385+273.15);

%% plot

eta_cross_fun_385 = eta_cross_fun(eta_fit_385,tau_fit_base,n_fit_base,gam_dot_cross_vec);

% plot data
figure(1), clf(1), hold on

% exp. data
plot(ang_freq,eta_380,'k^','markersize',6,'displayname',['Plate-plate meas. 380 $^\circ$C'],'MarkerIndices',1:2:length(ang_freq))
plot(ang_freq,eta_390,'ko','markersize',6,'displayname',['Plate-plate meas. 390 $^\circ$C'],'MarkerIndices',1:2:length(ang_freq))

% Cross model fit
plot(gam_dot_cross_vec,eta_cross_fun_385,'-k','linewidth',2.0,'displayname',['Cross model 385 $^\circ$C, $\eta_0$ = ', num2str(eta_fit_385,'%0.f'),' Pas,' newline '$\tau^*$ = ', num2str(tau_fit_base,'%0.3g'), ' Pa, $n$ = ', num2str(n_fit_base,'%0.2f')])

%% Figure options

xticks([1 1e1 1e2])
box on
grid on
legend('location','southwest','interpreter','latex','fontsize',8,'box','off')
xlabel('Shear rate, $\dot{\gamma}$ [1/s]','interpreter','latex','fontsize',8)
ylabel('Viscosity, $\eta$ [Pas]','interpreter','latex','fontsize',8)
set(gca,'XScale','log','YScale','log','fontsize',8)

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'units','centimeters','position',[5,5,width,0.75*width])
axis([5e-1 200 200 700])
grid on
box on
