clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Run certain height profile with 1D shear rate approach with White-Metzner
% to compute VE response with and without wall slip using state-rate
% model for slip evolution

% Version:
%   13-10-2022

% Matlab version: R2021a/R2022b

%% initialisation

mat_type = 1; %1: C/PEEK, 2: C/LM-PAEK
V_vec = [125 60 40 25 5]; % sliding velocity, mm/min (for modeling)
slip = 1; % 1: slip effect, else no slip condition
mode = 3; % 1: default, 2: constant rates (k_min = k_max), 3: approximate matrix interlayer thickness distribution through wider segments
dt = 0.01; % time step
tau_cr_vec = [35e3 55e3]; % critical shear stress for onset of strong slip
G_WM_vec = [500 400]; % parameter for White-Metzner (stiffness, controls start-up slope)

% state-rate model parameters and initial values
k_min_vec = [0.75 1.8]; 
a_slip_vec = [4.6 0.25]; 
m_slip_vec = [1.95 4.2];
Z_e_0 = 1.0;
Z_d_0 = 0.0;
Z_o_0 = 0.0;

% viscosity parameters for Cross model (from rheometry data)
eta_0_Cross_vec = [588, 499]; % zero shear viscosity
tau_0_Cross_vec = [6.05e4, 2.53e5]; 
n_Cross_vec = [0.38, 0.42];  
const_contr_vec = [1.8, 0.5]; % constant contribution (fiber-fiber interaction)

% figures
figure(1), clf(1), hold on
figure(2), clf(2), hold on

% parameters based on material type
G_WM = G_WM_vec(mat_type);                % WM parameter, start-up behavior
tau_cr = tau_cr_vec(mat_type);            % Slip parameter, onset for strong slip
k_min = k_min_vec(mat_type);              % State-rate parameter, minimum rate 
a_slip = a_slip_vec(mat_type);            % State-rate parameter, V-dependency
m_slip = m_slip_vec(mat_type);            % State-rate parameter, V-dependency
eta_0_Cross = eta_0_Cross_vec(mat_type);  % Cross model parameter
tau_0_Cross = tau_0_Cross_vec(mat_type);  % Cross model parameter
n_Cross = n_Cross_vec(mat_type);          % Cross model parameter
tau_y = const_contr_vec(mat_type);        % residual stress after full relaxation (not from VE response, yield stress)

% Cross model description
eta_func = @(V,h) ((eta_0_Cross)./(1+(eta_0_Cross.*(V./h)./tau_0_Cross).^(1-n_Cross)));

% find critical shear rate with viscous shear stress = tau_cr (for V_s,infty)
fun_find_gam = @(gam) ((eta_0_Cross)./(1+(eta_0_Cross.*(gam)./tau_0_Cross).^(1-n_Cross))).*(gam)-tau_cr; % function for gam_dot for which viscous response (Cross model) equals critical shear stress 
gam_dot_cr = fzero(fun_find_gam,tau_cr/eta_0_Cross*2); % solve for gam_dot

%% Plot measured friction response

% friction curves to call corresponding to material type
if mat_type == 1
    files_curves = ["curve_T_C-PEEK_p15_u125_T385_15-7","curve_T_C-PEEK_p15_u125_T385_15-8","curve_T_C-PEEK_p15_u125_T385_15-9","curve_T_C-PEEK_p15_u60_T385_15-18","curve_T_C-PEEK_p15_u40_T385_15-17","curve_T_C-PEEK_p15_u25_T385_2-1","curve_T_C-PEEK_p15_u25_T385_2-2","curve_T_C-PEEK_p15_u25_T385_2-3","curve_T_C-PEEK_p15_u25_T385_2-4","curve_T_C-PEEK_p15_u25_T385_2-5","curve_T_C-PEEK_p15_u5_T385_2-6","curve_T_C-PEEK_p15_u5_T385_3-7","curve_T_C-PEEK_p15_u5_T385_3-8"];
    figure(1), axis([0 9.5 0 100])
elseif mat_type == 2
    files_curves = ["curve_T_C-LM-PAEK_p15_u125_T365_17-10","curve_T_C-LM-PAEK_p15_u125_T365_17-11","curve_T_C-LM-PAEK_p15_u125_T365_17-12","curve_T_C-LM-PAEK_p15_u60_T365_17-34","curve_T_C-LM-PAEK_p15_u40_T365_17-31","curve_T_C-LM-PAEK_p15_u25_T365_17-7","curve_T_C-LM-PAEK_p15_u25_T365_17-8","curve_T_C-LM-PAEK_p15_u25_T365_17-9","curve_T_C-LM-PAEK_p15_u5_T365_17-4","curve_T_C-LM-PAEK_p15_u5_T365_17-5","curve_T_C-LM-PAEK_p15_u5_T365_17-6"];
    figure(1), axis([0 9.5 0 150])
else
    files_curves = [];
end

% plot curves (with average and std for repeated measurements)
V_i_prev = 0;
curves_store = [];
count_store = 0;

figure(1), hold on
for i = 1:length(files_curves) % go trough list of friction curves
    
    % load data
    cd ../Friction_curves
    load(files_curves(i));
    cd ../Scripts
    
    % find velocity (from file name)
    file_name_str = num2str(files_curves(i));
    idx_v = max(find(file_name_str=='u'));
    idx_v_end = idx_v + 1;
    while file_name_str(idx_v_end) ~= '_'
        idx_v_end = idx_v_end + 1;
    end 
    V_i = str2double(file_name_str(idx_v+1:idx_v_end-1)); % velocity
        
    if V_i == V_i_prev && i ~= length(files_curves) || isempty(curves_store) % gather friction curves with same sliding rate
        
        curves_store{count_store+1} = [d_save tau_save];
        count_store = count_store + 1;
        V_i_prev = V_i;
                
    else   % compute avg of stored curves and plot, change V_mean_prev and store current curves   
        
        % color codes for plotting
        if V_i_prev == 125
            color_code = [0.0 0.45 0.74];
        elseif V_i_prev == 60
            color_code = [0.85 0.33 0.1];
        elseif V_i_prev == 40
            color_code = [0.93 0.69 0.13];
        elseif V_i_prev == 25
            color_code = [0.49 0.18 0.56];
        elseif V_i_prev == 5
            color_code = [0.47 0.67 0.19];
        else
            color_code = [0 0 0];
        end
        
        if V_i == V_i_prev && i == length(files_curves)
            curves_store{count_store+1} = [d_save tau_save];
        end
        
        if length(curves_store) == 1 % no avg possible (only single measurement at considered velocity)
            
            % plot single curve
            plot(curves_store{1}(:,1),curves_store{1}(:,2),'-k','linewidth',1.0,'color',color_code,'displayname',[num2str(V_i_prev), ' mm/min'])
            
        else % compute avg for considered velocity
                               
            % find index for longest dataset in stored curves
            j_start = 1;
            j_end = length(curves_store);
            j_max = 1;
            length_max = 1;
            for j = 1:j_end
                if length(curves_store{j}) > length_max
                    j_max = j;
                    length_max = length(curves_store{j});
                end
            end

            % find start and end indices of basis curve (longest)
            d_base = curves_store{j_max}(:,1);
            k_start = min(find(d_base>0));
            d_end = 10;
            for j = 1:j_end % find shortest distance of curves
                if max(curves_store{j}(:,1)) < d_end
                    d_end = max(curves_store{j}(:,1));
                end
            end
            k_end = min(find(d_base>=d_end))-1;
            
            % determine mean curves
            d_curve = zeros(1,k_end-k_start+1);
            tau_curve = zeros(1,k_end-k_start+1);
            std_curve = zeros(1,k_end-k_start+1);
            k = k_start;
            while k <= k_end
                d_k = d_base(k);
                d_curve(k-k_start+1) = d_k;

                % find clostes tau values; interpolation
                tau_k = zeros(1,j_end);
                idx_zero = []; % account for end of curves; no data
                for j = 1:j_end
                    if j == j_max
                        tau_k(j) = curves_store{j}(k,2);
                    else
                        idx_left = max(find(curves_store{j}(:,1)<d_k));
                        idx_right = min(find(curves_store{j}(:,1)>d_k));
                        if ~isempty(idx_right)

                            % interpolation to find tau_k
                            tau_L = curves_store{j}(idx_left,2); 
                            tau_R = curves_store{j}(idx_right,2);
                            d_L = curves_store{j}(idx_left,1);
                            d_R = curves_store{j}(idx_right,1);
                            tau_k(j-j_start+1) = tau_L + (tau_R-tau_L)*((d_k-d_L)/(d_R-d_L));
                        else
                            idx_zero = [idx_zero, j];
                        end
                    end
                end

                tau_k(idx_zero) = [];
                tau_curve(k-k_start+1) = mean(tau_k);
                std_curve(k-k_start+1) = std(tau_k);
                k = k + 1;
            end
            
            % plot average and plot error band (standard deviation)
            plt = fill([d_curve, fliplr(d_curve)],[tau_curve-std_curve, fliplr(tau_curve+std_curve)],[0.5 0.5 0.5],'LineStyle','none','facecolor',color_code);
            set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
            set(plt, 'facealpha', 0.3)
            plot(d_curve,tau_curve,'-k','linewidth',1.0,'color',color_code,'displayname',[num2str(V_i_prev), ' mm/min']);     
        end
        
        % emtpy storage and store current curves or plot if last single curve       
        if V_i ~= V_i_prev && i == length(files_curves)
            plot(d_save,tau_save,'-k','linewidth',1.0,'displayname',[num2str(V_i), ' mm/min']);
        end
        curves_store = [];
        count_store = 1;
        V_i_prev = V_i;
        curves_store{count_store} = [d_save tau_save];
                
    end
end

%% Compute shear flow model with slip evolution

% choose ply-ply matrix interlayer thickness profiles
profile = "profile_gen_dist_59_multiple.mat"; % profile 3 shows average response (based on results in earlier paper for prediction friction characteristics)

% load profiles
load(profile);

% pick profile 3 (from 10 profiles in dataset)
x_profile = store_profile{3}(1,:)*1e3; % measured profile mm, width
y_profile = store_profile{3}(2,:)*1e3; % measured profile mm, height
if mode == 3 % approximate matrix interlayer thickness distribution through wider segments
    
    % sort profile with accounting for possible differences in x_profile
    dy_vec = zeros(1,length(x_profile)-1);
    dx_vec = zeros(1,length(x_profile)-1);
    
    % compute segments (as used in shear flow model)
    for i = 1:length(x_profile)-1
        dy_vec(i) = 0.5*(y_profile(i)+y_profile(i+1));
        dx_vec(i) = x_profile(i+1)-x_profile(i);
    end
    
    % sort segments
    [dy_sort,i_sort] = sort(dy_vec);
    dx_sort = dx_vec(i_sort);
    
    % sorted profile (for fit)
    y_profile_sort = dy_sort;
    x_profile_sort(1) = 0.5*dx_sort(1);
    for i = 2:length(dx_sort)
        x_profile_sort(i) = x_profile_sort(i-1) + 0.5*dx_sort(i-1) + 0.5*dx_sort(i);
    end
    
    % max length profile
    x_profile_max = x_profile_sort(end)+0.5*dx_sort(end);

    % plot sorted profile
    figure(2), hold on
    plot(x_profile_sort/x_profile_max,y_profile_sort*1e3,'-k','linewidth',1.0)
    
    % predefine segment width, calculate corresponding height based on equivalent area
    seg_width = 0.1; 
    area_i = 0; % count area of current segment
    dx_i = 0;
    seg_i = 1;
    seg_width_i = seg_width;
    x_profile_end_prev = 0;
    x_mid_profile_red = zeros(1,1/seg_width); % segments, for check and plot
    y_mid_profile_red = zeros(1,1/seg_width); % segments, for check and plot
    x_profile_red = zeros(1,length(x_mid_profile_red)+1); % profile, points at edges of segment
    y_profile_red = zeros(1,length(x_mid_profile_red)+1); % profile, points at edges of segment
    y_profile_red(1) = dy_sort(1); % take smallest dy in profile as initial height
    
    % go through sorted profile
    for i = 1:length(x_profile_sort)
        if x_profile_sort(i)/x_profile_max <= seg_width_i  % within current segment
            area_i = area_i + dx_sort(i)*dy_sort(i)/x_profile_max;
            dx_i = dx_i + dx_sort(i)/x_profile_max;
        else  % end of segment; compute height and start with new segment
            x_mid_profile_red(seg_i) = x_profile_red(seg_i) + 0.5*dx_i;
            x_profile_red(seg_i+1) = x_profile_red(seg_i) + dx_i;
            y_mid_profile_red(seg_i) = area_i/dx_i;
            y_profile_red(seg_i+1) = y_profile_red(seg_i) + 2*(y_mid_profile_red(seg_i)-y_profile_red(seg_i));
            area_i = dx_sort(i)*dy_sort(i)/x_profile_max;
            dx_i = dx_sort(i)/x_profile_max;
            seg_width_i = seg_width_i + seg_width;
            seg_i = seg_i + 1;                    
        end
    end

    % finalize last segment
    x_mid_profile_red(seg_i) = x_profile_red(seg_i) + 0.5*dx_i;
    x_profile_red(seg_i+1) = x_profile_red(seg_i) + dx_i;
    y_mid_profile_red(seg_i) = area_i/dx_i;
    y_profile_red(seg_i+1) = y_profile_red(seg_i) + 2*(y_mid_profile_red(seg_i)-y_profile_red(seg_i));

    % use reduced points as profile
    x_profile = x_profile_red;
    y_profile = y_profile_red;
               
    % plot segments
    y_i_prev = 0;
    alpha_color = 0.6; % for grey scale of segment lines
    for i = 1:length(x_profile_red)-1
        y_i = 0.5*(y_profile_red(i+1) + y_profile_red(i));
        plot([x_profile_red(i) x_profile_red(i)],[0 y_i*1e3],'-k','linewidth',1.0,'color',alpha_color*[1 1 1])
        plot([x_profile_red(i) x_profile_red(i+1)],[y_i*1e3 y_i*1e3],'-k','linewidth',1.0,'color',alpha_color*[1 1 1])
        %plot(x_mid_profile_red(i),y_mid_profile_red(i),'rx','markersize',12)
        y_i_prev = y_i;
    end
    y_i = 0.5*(y_profile_red(end) + y_profile_red(end-1));
    plot([x_profile_red(end-1) x_profile_red(end-1)],[y_i_prev*1e3 y_i*1e3],'-k','linewidth',1.0,'color',alpha_color*[1 1 1])
    plot([x_profile_red(end-1) x_profile_red(end)],[y_i*1e3 y_i*1e3],'-k','linewidth',1.0,'color',alpha_color*[1 1 1])
    plot([x_profile_red(end) x_profile_red(end)],[y_i*1e3 0],'-k','linewidth',1.0,'color',alpha_color*[1 1 1]) 
end

for j = 1:length(V_vec) % go through velocities
    V_sliding = V_vec(j)/60; % mm/s
    t_e = 10/V_sliding; % end time for numerical time integration
    t = 0:dt:t_e;  % time vector
    d = t*V_sliding; % displacement vector
    tau_store = zeros(length(x_profile)-1,length(t)); % store tau curves per width increment
    slip_count = 0; % for percentage of slip (at certain velocity)
    
    % go through ply-ply matrix interlayer thickness profile
    for i = 1:length(x_profile)-1
        dx = x_profile(i+1)-x_profile(i); % width of small segment, used for weighting if not equidistant
        dy = 0.5*(y_profile(i)+y_profile(i+1)); % height of small segment
        if dy == 0 % no shear rate, thus zero shear stress
            tau_i = zeros(1,length(t));
        else
            
            % White-Metzner DE for Runge-Kutta numerical time integration
            f = @(tau,Vs) (G_WM.*((V_sliding-2*Vs)/dy) - (G_WM./eta_func(V_sliding-2*Vs,dy)).*tau);
           
            % initialisation
            tau_i = zeros(1,length(t)); % store shear stresses
            Vs = zeros(1,length(t)); % store slip velocities
            strong_slip = 0;
            Z_e = ones(1,length(t)).*Z_e_0;  % store average state of active chains (or entangled)
            Z_d = ones(1,length(t)).*Z_d_0;  % store average state of disentangled chains
            Z_o = ones(1,length(t)).*Z_o_0;  % store average state of steady chains (or oriented) 

            for k = 1:length(t)-1  % go through time vector (within small segment of ply-ply interface)        

                % Runge-Kutta for numerical time integration
                k1 = f(tau_i(k),Vs(k));
                k2 = f(tau_i(k)+0.5*dt*k1,Vs(k));
                k3 = f(tau_i(k)+0.5*dt*k2,Vs(k));
                k4 = f(tau_i(k)+dt*k3,Vs(k));
                
                % shear stress at next time step
                tau_i(k+1) = tau_i(k) + (1/6)*dt*(k1+2*k2+2*k3+k4);
                
                % strong slip effect (with slip evolution)
                if (slip == 1 && tau_i(k) >= tau_cr) || strong_slip == 1
                    strong_slip = 1;
                    gam_dot = V_sliding/dy; % local shear rate in small segment
                  
                    % stationary slip velocity (V_s,infty denoted here a Vs_cr) such that true shear rates results in critcal shear rate                    
                    Vs_cr = 0.5*(V_sliding-gam_dot_cr*dy); % max slip velocity (to get tau_cr)
                    Vs_cr  = max(Vs_cr,0); % only if gam_dot > gam_dot_cr!
                   
                    % rates (based on mode and applied rate) 
                    if mode == 2
                        k_max = k_min;
                    else
                        k_max = a_slip.*((V_sliding)).^m_slip + k_min; % max rate
                    end
                    k_d = k_min + (k_max-k_min)*(1-Z_e(k)); % disentangling rate
                    k_e = k_max - (k_max-k_min)*(1-Z_e(k)); % reentanglement rate
                    k_o = k_d; % conformation or orientation rate
                    
                    % integration of states (based on rates, forward Euler)
                    Z_e(k+1) = Z_e(k) - k_d*Z_e(k)*dt + k_e*Z_d(k)*dt;   
                    Z_d(k+1) = Z_d(k) + k_d*Z_e(k)*dt - k_e*Z_d(k)*dt - k_o*Z_d(k)*dt;
                    Z_o(k+1) = Z_o(k) + k_o*Z_d(k)*dt;
                    
                    % safety: states within 0 and 1
                    Z_e(k+1) = min(Z_e(k+1),1);
                    Z_d(k+1) = min(Z_d(k+1),1);
                    Z_o(k+1) = min(Z_o(k+1),1);
                    Z_e(k+1) = max(Z_e(k+1),0);
                    Z_d(k+1) = max(Z_d(k+1),0);
                    Z_o(k+1) = max(Z_o(k+1),0);

                    % slip velocity: based on average steady (or orientated) state with prefactor (stationary slip velocity)
                    Vs(k+1) = Z_o(k+1)*Vs_cr;
                    
                end
            end
            
        end
        tau_store(i,:) = tau_i*dx; %shear stress small portion times width(weighting), [Pamm]
        
        if strong_slip == 1
            slip_count = slip_count + 1; % for precentage of slip
        end       
    end
    tau_avg = sum(tau_store,1)/(x_profile(end)-x_profile(1));
    
    % color codes depending on velocity
    if V_vec(j) == 125
        color_code = [0.0 0.45 0.74];
    elseif V_vec(j) == 60
        color_code = [0.85 0.33 0.1];
    elseif V_vec(j) == 40
        color_code = [0.93 0.69 0.13];
    elseif V_vec(j) == 25
        color_code = [0.49 0.18 0.56];
    elseif V_vec(j) == 5
        color_code = [0.47 0.67 0.19];
    else
        color_code = [0 0 0];
    end
    
    figure(1), hold on
    plt = plot(d,tau_avg*1e-3+tau_y,'--','displayname',['Model'],'linewidth',1.0,'color',color_code); % plot VE response + constant contribution (yield stress)
    set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
    
    fprintf(['Perc. slip: ',num2str(slip_count/(length(x_profile)-1)*100),'\n'])
    
end

%% Figure options

figure(1)
if mat_type == 1 && mode == 1
    text(0.2,95,'C/PEEK','interpreter','latex','fontsize',8)
elseif mat_type == 2 && mode == 1
    text(0.2,140,'C/LM-PAEK','interpreter','latex','fontsize',8)
end

% use dummy lines to label model in legend
if mode == 1
    plot([10 11],[0 0],'--k','linewidth',1.0,'displayname',['Model'])
    legend('location','northeast','interpreter','latex','fontsize',8,'box','off')
elseif mode == 2
    plot([10 11],[0 0],'--k','linewidth',1.0,'displayname',['Model,' newline '$k_i$ = ', num2str(k_min,'%.2f')])
    legend('box','on','edgecolor','w','interpreter','latex','fontsize',8,'location','NorthEast')
elseif mode == 3
    plot([10 11],[0 0],'--k','linewidth',1.0,'displayname',['Model,' newline '10 segments'])
    legend('location','northeast','interpreter','latex','fontsize',8,'box','off')
end
xlabel('Displacement, $d$ [mm]','interpreter','latex','fontsize',8)
ylabel('Shear stress, $\tau$ [kPa]','interpreter','latex','fontsize',8)
set(gca,'fontsize',8)

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'renderer','painters','units','centimeters','position',[5,5,width,0.75*width])
grid on
box on

if mode == 3 % plot matrix interlayer thickness distribution (profile) and segments (coarse approximation of distribution)
    figure(2)
    xlabel('Normalized width, $x/w$ [-]','interpreter','latex','fontsize',8)
    ylabel('Matrix interlayer thickness, $h(x)$ [$\mu$m]','interpreter','latex','fontsize',8)
    legend('Sorted distribution','Segments','location','northwest','interpreter','latex','fontsize',8,'box','off')
    set(gca,'fontsize',8)

    % figure size
    width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
    set(gcf,'renderer','painters','units','centimeters','position',[5,5,width,0.75*width])
    grid on
    box on
end

%% analytical solutions
%{
Z_e_fun = @(k,t) (-1)/(sqrt(5))*exp(0.5*k*(-3-sqrt(5))*t)*(0.5*(1-sqrt(5))) + (1)/(sqrt(5))*exp(0.5*k*(-3+sqrt(5))*t)*(0.5*(1+sqrt(5)));
Z_d_fun = @(k,t) (-1)/(sqrt(5))*exp(0.5*k*(-3-sqrt(5))*t) + (1)/(sqrt(5))*exp(0.5*k*(-3+sqrt(5))*t);
Z_o_fun = @(k,t) (-exp(0.5*k*(-3-sqrt(5))*t))/(0.5*(-3*sqrt(5)-5)) + (exp(0.5*k*(-3+sqrt(5))*t))/(0.5*(-3*sqrt(5)+5)) + 1;

Z_e_vec = Z_e_fun(k_min,t);
Z_d_vec = Z_d_fun(k_min,t);
Z_o_vec = Z_o_fun(k_min,t);

figure(6), hold on
plot(d+d_corr,Z_o_vec*Vs_cr_plot,'--r','linewidth',2.0)

figure(8), hold on
plot(d+d_corr,Z_e_vec,'--k','linewidth',2.0)
plot(d+d_corr,Z_d_vec,'--r','linewidth',2.0)
plot(d+d_corr,Z_o_vec,'--b','linewidth',2.0)
%}


